import os
import json
from deepsecurity.rest import ApiException


def value_exists(some_json, key):
    value = some_json.get(key)
    if value is not None:
        return value
    else:
        return False


def rename_json(json):
    if json["name"]:
        json["name"] = f"{json['name']} - Migrated"
    return json


def validate_create(all_old, api_instance, type):
    all_new = []
    for count, dirlist in enumerate(all_old):
        namecheck = 1
        rename = 1
        oldjson = json.loads(dirlist)
        oldname = oldjson["name"]
        while namecheck != -1:
            try:
                newname = api_instance.create(oldjson)
                newid = api_instance.search(newname)
                print(
                    "#"
                    + str(count)
                    + " "
                    + type.capitalize()
                    + " List ID: "
                    + str(newid)
                    + ", Name: "
                    + newname,
                    flush=True,
                )
                all_new.append(str(newid))
                namecheck = -1
            except ApiException as e:
                error_json = json.loads(e.body)
                if (
                    "requested scheduled task name already exists"
                    in error_json["message"]
                ):
                    oldjson["name"] = oldname + " {" + str(rename) + "}"
                    rename = rename + 1
                elif (
                    "name already exists"
                    or "Name must be unique" in error_json["message"]
                ):
                    print(
                        f"{oldjson['name']} already exists in new tenant, renaming...",
                        flush=True,
                    )
                    oldjson["name"] = oldname + " {" + str(rename) + "}"
                    rename = rename + 1
                else:
                    print(e.body, flush=True)
                    namecheck = -1
    return all_new


def validate_create_dict(all_old, api_instance, type):
    new_dict = {}
    for count, dirlist in enumerate(all_old):
        namecheck = 1
        rename = 1
        oldjson = json.loads(dirlist)
        oldname = oldjson["name"]
        oldid = oldjson["ID"]
        while namecheck != -1:
            try:
                newname = api_instance.create(oldjson)
                newid = api_instance.search(newname)
                print(
                    "#"
                    + str(count)
                    + " "
                    + type.capitalize()
                    + " List ID: "
                    + str(newid)
                    + ", Name: "
                    + newname,
                    flush=True,
                )
                new_dict[oldid] = newid
                namecheck = -1
            except ApiException as e:
                error_json = json.loads(e.body)
                if "name already exists" in error_json["message"]:
                    print(
                        f"{oldjson['name']} already exists in new tenant, renaming..."
                    )
                    oldjson["name"] = oldname + " {" + str(rename) + "}"
                    rename = rename + 1
                else:
                    print(e.body, flush=True)
                    namecheck = -1
    return new_dict



import sys
import os
import deepsecurity
from deepsecurity.rest import ApiException
import time
from time import sleep
import requests
import urllib3
import json
from dsmigrator.api_config import PolicyApiInstance
from rich import print
import click

cert = False


def ListAllPolicy(url_link_final, tenant1key):
    payload = {}
    url = url_link_final + "api/policies"
    headers = {
        "api-secret-key": tenant1key,
        "api-version": "v1",
        "Content-Type": "application/json",
    }
    response = requests.request("GET", url, headers=headers, data=payload, verify=cert)
    describe = str(response.text)
    index = 0
    oldpolicyname = []
    oldpolicyid = []
    namejson = json.loads(describe)
    for policy in namejson["policies"]:
        oldpolicyname.append(str(policy["name"]))
        oldpolicyid.append(str(policy["ID"]))
    return enumerate(oldpolicyname), oldpolicyid


def GetPolicy(policyIDs, url_link_final, tenant1key):
    antimalwareconfig = []
    allofpolicy = []
    i = 0
    print("Getting Policy from Tenant 1", flush=True)
    for count, part in enumerate(policyIDs):

        payload = {}
        url = url_link_final + "api/policies/" + str(part)
        headers = {
            "api-secret-key": tenant1key,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "GET", url, headers=headers, data=payload, verify=cert
        )

        describe = str(response.text)
        i = i + 1
        allofpolicy.append(describe)
        print("#" + str(count) + " Policy ID: " + str(part), flush=True)
        rtscan = describe.find("realTimeScanConfigurationID")
        if rtscan != -1:
            rtpart = describe[rtscan + 28 :]
            startIndex = rtpart.find(":")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = rtpart.find(",", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    rtid = rtpart[startIndex + 1 : endIndex]
                    antimalwareconfig.append(str(rtid))

        mscan = describe.find("manualScanConfigurationID")
        if mscan != -1:
            mpart = describe[mscan + 26 :]
            startIndex = mpart.find(":")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = mpart.find(",", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    mid = mpart[startIndex + 1 : endIndex]
                    antimalwareconfig.append(str(mid))

        sscan = describe.find("scheduledScanConfigurationID")
        if sscan != -1:
            spart = describe[sscan + 29 :]
            startIndex = spart.find(":")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = spart.find("}", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    ssid = spart[startIndex + 1 : endIndex]
                    antimalwareconfig.append(str(ssid))
    antimalwareconfig = list(dict.fromkeys(antimalwareconfig))
    return antimalwareconfig, allofpolicy


def validate_create(all_old, api_instance, type):
    all_new = []
    id_dict = {}
    for count, dirlist in enumerate(all_old):
        namecheck = 1
        rename = 1
        oldjson = json.loads(dirlist)
        oldname = oldjson["name"]
        oldid = oldjson["ID"]
        while namecheck != -1:
            if "parentID" in oldjson.keys():
                newparentid = id_dict[oldjson["parentID"]]
            try:
                newname = api_instance.create(oldjson)
                newid = api_instance.search(newname).id
                id_dict[oldid] = newid
                if "parentID" in oldjson.keys():
                    api_instance.modify_parent(newid, newparentid)
                print(
                    "#"
                    + str(count)
                    + " "
                    + type.capitalize()
                    + " List ID: "
                    + str(newid)
                    + ", Name: "
                    + newname,
                    flush=True,
                )
                all_new.append(str(newid))
                namecheck = -1
            except ApiException as e:
                error_json = json.loads(e.body)
                if "name already exists" in error_json["message"]:
                    print(
                        f"{oldjson['name']} already exists in new tenant, renaming..."
                    )
                    oldjson["name"] = oldname + " {" + str(rename) + "}"
                    rename = rename + 1
                else:
                    print(e.body)
                    print(
                        f"[bold bright_red]WARNING[/bold bright_red]: {oldname} could not be transferred. Please transfer manually."
                    )
                    print(
                        "[i]NOTE: This is under construction and will work soon. :)[/i]"
                    )
                    namecheck = -1
    return id_dict


cert = False


def AddPolicy(allofpolicy, NEW_API_KEY):
    print("Creating Policy to Tenant 2 with new ID", flush=True)
    if allofpolicy:
        return validate_create(allofpolicy, PolicyApiInstance(NEW_API_KEY), "policy")



import sys
from rich.console import Console
import os
import time
from time import sleep
import requests
import urllib3
import json
from deepsecurity.rest import ApiException
from dsmigrator.api_config import AMConfigApiInstance
from dsmigrator.migrator_utils import validate_create, validate_create_dict

console = Console()


cert = False


def am_config_transform(antimalwareconfig, OLD_HOST, OLD_API_KEY):
    amdirectorylist, amfileextentionlist, amfilelist, allamconfig = AMconfigtenant1(
        antimalwareconfig, OLD_HOST, OLD_API_KEY
    )
    return amdirectorylist, amfileextentionlist, amfilelist, allamconfig


def am_validate_create(
    allofpolicy,
    antimalwareconfig,
    allamconfig,
    amdirectorylist,
    amalldirectorynew,
    amfileextentionlist,
    amallfileextentionnew,
    amfilelist,
    amallfilelistnew,
    NEW_HOST,
    NEW_API_KEY,
):
    mod_allamconfig = AmConfigCheck(
        allamconfig,
        amdirectorylist,
        amalldirectorynew,
        amfileextentionlist,
        amallfileextentionnew,
        amfilelist,
        amallfilelistnew,
    )

    # renamed_allamconfig = RenameAmConfig(mod_allamconfig)

    allamconfigdict = AmconfigTenant2(mod_allamconfig, NEW_API_KEY)

    aop_replace_am_configs = AmReplaceConfig(allofpolicy, allamconfigdict)
    final = aop_replace_am_configs
    return final


def json_to_list(json_item, key, outlist):
    item = json_item.get(key)
    if item:
        outlist.append(str(item))
        return outlist


def AMconfigtenant1(antimalwareconfig, url_link_final, tenant1key):
    allamconfig = []
    directorylist = []
    fileextentionlist = []
    filelist = []
    print("Getting Anti-Malware configuration from Tenant 1", flush=True)
    for count, amconfig in enumerate(antimalwareconfig):
        if int(amconfig) != 0:
            payload = {}
            url = url_link_final + "api/antimalwareconfigurations/" + str(amconfig)
            headers = {
                "api-secret-key": tenant1key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "GET", url, headers=headers, data=payload, verify=cert
            )

            describe = str(response.text)
            allamconfig.append(describe)
            describe_json = json.loads(describe)
            json_to_list(describe_json, "directoryListID", directorylist)
            json_to_list(describe_json, "excludedDirectoryListID", directorylist)
            json_to_list(
                describe_json, "excludedFileExtensionListID", fileextentionlist
            )
            json_to_list(describe_json, "fileExtensionListID", fileextentionlist)
            json_to_list(describe_json, "excludedFileListID", filelist)
            json_to_list(describe_json, "fileListID", filelist)
            json_to_list(describe_json, "excludedProcessImageFileListID", filelist)
            print(
                "#" + str(count) + " Anti-Malware Config ID: " + str(amconfig),
                flush=True,
            )
    directorylist = list(dict.fromkeys(directorylist))
    fileextentionlist = list(dict.fromkeys(fileextentionlist))
    filelist = list(dict.fromkeys(filelist))
    return directorylist, fileextentionlist, filelist, allamconfig


def AmconfigTenant2(allamconfig, NEW_API_KEY):
    print("Creating Anti-Malware Configuration in Tenant2", flush=True)
    if allamconfig:
        allamconfigdict = validate_create_dict(
            allamconfig, AMConfigApiInstance(NEW_API_KEY), "anti malware"
        )
    print("New AM Config Dict", flush=True)
    print(allamconfigdict, flush=True)
    allamconfigdict[0] = 0
    return allamconfigdict


def AmReplaceConfig(allofpolicy, allamconfigdict):

    count = 0
    for describe in allofpolicy:

        def replace_am_id(
            type,
            policy_json_string,
            newiddict=allamconfigdict,
        ):
            policy_json = json.loads(policy_json_string)
            policy_am_json = policy_json["antiMalware"]
            oldid = policy_am_json[type]
            policy_am_json[type] = newiddict[oldid]
            final_json = json.dumps(policy_json)
            return final_json

        describe_rt = replace_am_id("realTimeScanConfigurationID", describe)
        describe_man = replace_am_id("manualScanConfigurationID", describe_rt)
        describe_sch = replace_am_id("scheduledScanConfigurationID", describe_man)

        allofpolicy[count] = describe_sch
        count = count + 1
    return allofpolicy


def AmConfigCheck(
    allamconfig,
    directorylist,
    alldirectorynew,
    fileextentionlist,
    allfileextentionnew,
    filelist,
    allfilelistnew,
):

    count = 0
    for describe in allamconfig:
        count1 = 0
        index = describe.find('"directoryListID"')
        if index != -1:
            indexpart = describe[index + 17 :]
            startIndex = indexpart.find(":")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = indexpart.find(",", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    indexid = indexpart[startIndex + 1 : endIndex]
                    for dirlist in directorylist:
                        if indexid == dirlist:
                            if alldirectorynew:
                                describe = (
                                    describe[: index + 17 + startIndex + 1]
                                    + alldirectorynew[count1]
                                    + describe[index + 17 + startIndex + endIndex :]
                                )
                        count1 = count1 + 1
        count1 = 0
        index = describe.find("excludedDirectoryListID")
        if index != -1:
            indexpart = describe[index + 24 :]
            startIndex = indexpart.find(":")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = indexpart.find(",", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    indexid = indexpart[startIndex + 1 : endIndex]
                    for dirlist in directorylist:
                        if indexid == dirlist:
                            if alldirectorynew:
                                describe = (
                                    describe[: index + 24 + startIndex + 1]
                                    + alldirectorynew[count1]
                                    + describe[index + 24 + startIndex + endIndex :]
                                )
                        count1 = count1 + 1
        count1 = 0
        index = describe.find("excludedFileExtensionListID")
        if index != -1:
            indexpart = describe[index + 28 :]
            startIndex = indexpart.find(":")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = indexpart.find(",", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    indexid = indexpart[startIndex + 1 : endIndex]
                    for dirlist in fileextentionlist:
                        if indexid == dirlist:
                            if allfileextentionnew:
                                describe = (
                                    describe[: index + 28 + startIndex + 1]
                                    + allfileextentionnew[count1]
                                    + describe[index + 28 + startIndex + endIndex :]
                                )
                        count1 = count1 + 1
        count1 = 0
        index = describe.find('"fileExtensionListID"')
        if index != -1:
            indexpart = describe[index + 21]
            startIndex = indexpart.find(":")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = indexpart.find(",", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    indexid = indexpart[startIndex + 1 : endIndex]
                    for dirlist in fileextentionlist:
                        if indexid == dirlist:
                            if allfileextentionnew:
                                describe = (
                                    describe[: index + 21 + startIndex + 1]
                                    + allfileextentionnew[count1]
                                    + describe[index + 21 + startIndex + endIndex :]
                                )
                        count1 = count1 + 1
        count1 = 0
        index = describe.find("excludedFileListID")
        if index != -1:
            indexpart = describe[index + 19 :]
            startIndex = indexpart.find(":")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = indexpart.find(",", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    indexid = indexpart[startIndex + 1 : endIndex]
                    for dirlist in filelist:
                        if indexid == dirlist:
                            if allfilelistnew:
                                describe = (
                                    describe[: index + 19 + startIndex + 1]
                                    + allfilelistnew[count1]
                                    + describe[index + 19 + startIndex + endIndex :]
                                )
                        count1 = count1 + 1
        count1 = 0
        index = describe.find("excludedProcessImageFileListID")
        if index != -1:
            indexpart = describe[index + 31 :]
            startIndex = indexpart.find(":")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = indexpart.find(",", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    indexid = indexpart[startIndex + 1 : endIndex]
                    for dirlist in filelist:
                        if indexid == dirlist:
                            if allfilelistnew:
                                describe = (
                                    describe[: index + 31 + startIndex + 1]
                                    + allfilelistnew[count1]
                                    + describe[index + 31 + startIndex + endIndex :]
                                )
                        count1 = count1 + 1
        allamconfig[count] = describe
        count = count + 1
        # console.print(f"CONFIGCHECKALLAMCONFIG:\n{allamconfig}", style="bold red")
    return allamconfig


def RenameAmConfig(allamconfig):
    count = 0
    if allamconfig:
        for describe in allamconfig:
            amjson = json.loads(describe)
            print(describe)
            amjson["name"] = amjson["name"] + " - Migrated"
            allamconfig[count] = json.dumps(amjson)
            count = count + 1
    # console.print(f"RENAMEALLAMCONFIG:\n{allamconfig}", style="bold red")
    return allamconfig



import sys
import os
import time
from time import sleep
import requests
import urllib3
import json

cert = False


def im_config_transform(allofpolicy, OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY):
    imruleid = IMGet(allofpolicy)

    allimrule, allimruleidnew1, allimruleidold, allimcustomrule = IMDescribe(
        imruleid, OLD_HOST, OLD_API_KEY, NEW_HOST, NEW_API_KEY
    )

    allimruleidnew2 = IMCustom(allimrule, allimcustomrule, NEW_HOST, NEW_API_KEY)
    aop_replace_im_rules = IMReplace(
        allofpolicy,
        allimruleidnew1,
        allimruleidnew2,
        imruleid,
        allimruleidold,
        allimcustomrule,
    )
    final = aop_replace_im_rules
    return final


def IMGet(allofpolicy):
    imruleid = []
    print("IM rules in Tenant 1", flush=True)
    for describe in allofpolicy:
        namejson = json.loads(describe)
        if "ruleIDs" in namejson["integrityMonitoring"]:
            for count, here2 in enumerate(namejson["integrityMonitoring"]["ruleIDs"]):
                imruleid.append(str(here2))
    imruleid = list(dict.fromkeys(imruleid))
    print(imruleid, flush=True)
    return imruleid


def IMDescribe(imruleid, url_link_final, tenant1key, url_link_final_2, tenant2key):
    allimrule = []
    allimrulename = []
    allimruleidnew1 = []
    allimruleidold = []
    allimcustomrule = []
    print("Searching IM rules in Tenant 1...", flush=True)
    if imruleid:
        for count, dirlist in enumerate(imruleid):
            payload = {}
            url = url_link_final + "api/integritymonitoringrules/" + str(dirlist)
            headers = {
                "api-secret-key": tenant1key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "GET", url, headers=headers, data=payload, verify=cert
            )
            describe = str(response.text)
            allimrule.append(describe)
            imjson = json.loads(describe)
            allimrulename.append(str(imjson["name"]))
            print(
                "#" + str(count) + " IPS rule name: " + str(imjson["name"]), flush=True
            )
            print("#" + str(count) + " IM rule ID: " + str(dirlist), flush=True)
    print("Done!", flush=True)
    print("Searching and Modifying IM rule in Tenant 2...", flush=True)
    for count, dirlist in enumerate(allimrulename):
        payload = (
            '{"searchCriteria": [{"fieldName": "name","stringValue": "'
            + dirlist
            + '"}]}'
        )
        url = url_link_final_2 + "api/integritymonitoringrules/search"
        headers = {
            "api-secret-key": tenant2key,
            "api-version": "v1",
            "Content-Type": "application/json",
        }
        response = requests.request(
            "POST", url, headers=headers, data=payload, verify=cert
        )
        describe = str(response.text)
        taskjson = json.loads(describe)
        if not "message" in taskjson:
            index = describe.find(dirlist)
            if index != -1:
                index = describe.find('"ID"')
                if index != -1:
                    indexpart = describe[index + 4 :]
                    startIndex = indexpart.find(":")
                    if startIndex != -1:  # i.e. if the first quote was found
                        endIndex = indexpart.find(",", startIndex + 1)
                        if (
                            startIndex != -1 and endIndex != -1
                        ):  # i.e. both quotes were found
                            indexid = indexpart[startIndex + 1 : endIndex]
                            allimruleidnew1.append(str(indexid))
                            allimruleidold.append(count)
                            print(
                                "#" + str(count) + " IM rule ID: " + str(indexid),
                                flush=True,
                            )
                        else:
                            endIndex = indexpart.find("}", startIndex + 1)
                            if (
                                startIndex != -1 and endIndex != -1
                            ):  # i.e. both quotes were found
                                indexid = indexpart[startIndex + 1 : endIndex]
                                allimruleidnew1.append(str(indexid))
                                allimruleidold.append(count)
                                print(
                                    "#" + str(count) + " IM rule ID: " + str(indexid),
                                    flush=True,
                                )
                else:
                    print(describe, flush=True)
                    print(payload, flush=True)
            else:
                allimcustomrule.append(count)
        else:
            print(describe, flush=True)
            print(payload, flush=True)
    print("Done!", flush=True)
    return allimrule, allimruleidnew1, allimruleidold, allimcustomrule


def IMCustom(allimrule, allimcustomrule, url_link_final_2, tenant2key):
    allimruleidnew2 = []
    if allimcustomrule:
        print("Creating new custom IM rule in Tenant 2...", flush=True)
        for count, indexnum in enumerate(allimcustomrule):
            payload = allimrule[indexnum]
            url = url_link_final_2 + "api/integritymonitoringrules"
            headers = {
                "api-secret-key": tenant2key,
                "api-version": "v1",
                "Content-Type": "application/json",
            }
            response = requests.request(
                "POST", url, headers=headers, data=payload, verify=cert
            )
            describe = str(response.text)
            index = describe.find('"ID"')
            if index != -1:
                indexpart = describe[index + 4 :]
                startIndex = indexpart.find(":")
                if startIndex != -1:  # i.e. if the first quote was found
                    endIndex = indexpart.find("}", startIndex + 1)
                    if (
                        startIndex != -1 and endIndex != -1
                    ):  # i.e. both quotes were found
                        indexid = indexpart[startIndex + 1 : endIndex]
                        allimruleidnew2.append(str(indexid))
                        print(
                            "#" + str(count) + " IM rule ID: " + str(indexid),
                            flush=True,
                        )
            else:
                print(describe, flush=True)
                print(payload, flush=True)
        print("Done!", flush=True)

    return allimruleidnew2


def IMReplace(
    allofpolicy,
    allimruleidnew1,
    allimruleidnew2,
    imruleid,
    allimruleidold,
    allimcustomrule,
):
    for count, describe in enumerate(allofpolicy):
        index = describe.find('"integrityMonitoring"')
        if index != -1:
            indexpart = describe[index + 20 :]
            startIndex = indexpart.find("}")
            if startIndex != -1:  # i.e. if the first quote was found
                endIndex = indexpart.find("}", startIndex + 1)
                if startIndex != -1 and endIndex != -1:  # i.e. both quotes were found
                    indexid = indexpart[startIndex + 1 : endIndex]
                    index2 = indexid.find("ruleIDs")
                    if index2 != -1:
                        indexpart2 = indexid[index2 + 7 :]
                        startIndex2 = indexpart2.find("[")
                        if startIndex2 != -1:  # i.e. if the first quote was found
                            endIndex2 = indexpart2.find("]", startIndex2 + 1)
                            if (
                                startIndex2 != -1 and endIndex2 != -1
                            ):  # i.e. both quotes were found
                                indexid2 = indexpart2[startIndex2 + 1 : endIndex2]
                                indexid3 = indexpart2[startIndex2 + 1 : endIndex2]
                                indexid4 = indexid2.split(", ")
                                if allimruleidnew1 or allimruleidnew2:
                                    for count1, this in enumerate(indexid4):
                                        checkindex = imruleid.index(this)
                                        if checkindex in allimruleidold:
                                            checkindex1 = allimruleidold.index(
                                                checkindex
                                            )
                                            indexid4[count1] = allimruleidnew1[
                                                checkindex1
                                            ]
                                        elif checkindex in allimcustomrule:
                                            checkindex1 = allimcustomrule.index(
                                                checkindex
                                            )
                                            indexid4[count1] = allimruleidnew2[
                                                checkindex1
                                            ]
                                    indexid2 = ",".join(indexid4)
                                modulepart = describe[index : index + 20 + endIndex]
                                modulepart2 = modulepart.replace(indexid3, indexid2)
                                allofpolicy[count] = describe.replace(
                                    modulepart, modulepart2
                                )
    return allofpolicy
